﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Drawing;
using System.IO;

using ProtoBuf;

using Vintagestory.API.Common;
using Vintagestory.API.Datastructures;

namespace Automap
{
	[ProtoContract(SkipConstructor = true)]
	//[ProtoInclude(5,typeof(DelegateState))]
	public class CommandData : IAttribute
	{
		[ProtoMember(1)]
		public RunState State { get; set; }//Run , Stop, SingleSnapshot -> Stop

		[ProtoMember(2)]
		private List<DelegateState> DelegatesFlags;

		[ProtoMember(3)]
		public string Notation { get; set; }

		//POI Delegate list {enabled/Disable}, color?
		//Other params...? Tick rate?
		//Choose : Renderer(s)


		public CommandData(RunState assumeState, bool[] theseDelegates)
		{
			State = assumeState;

			DelegatesFlags = new List<DelegateState>(theseDelegates.Length);
			foreach (var df in theseDelegates)
			{
				DelegatesFlags.Add(new DelegateState()
				{
					Enabled = df,
					AlternateColor = null,
				});

			}

		}

		public CommandData(RunState assumeState)
		{
			State = assumeState;//Never RUN.

			DelegatesFlags = new List<DelegateState>();
		}


		public void FromBytes(BinaryReader stream)
		{
			var temp = ProtoBuf.Serializer.Deserialize<CommandData>(stream.BaseStream);
			this.State = temp.State;
			this.DelegatesFlags = temp.DelegatesFlags;

		}

		public void ToBytes(BinaryWriter stream)
		{
			ProtoBuf.Serializer.Serialize<CommandData>(stream.BaseStream, this);


		}



		public int GetAttributeId()
		{
			return 12346;
		}

		public object GetValue()
		{
			return this;
		}


		public string ToJsonToken()
		{
			return $"New-State:{State}, Delegates# {DelegatesFlags.Count} ";
		}

		public bool Equals(IWorldAccessor worldForResolve, IAttribute attr)
		{
			var other = attr.GetValue() as CommandData;

			if (this.State == other.State)
			{
				return true;
			}

			return false;
		}
	}

	[ProtoContract]
	internal struct DelegateState
	{
		[ProtoMember(1)]
		public bool Enabled;

		[ProtoMember(2)]
		public Color? AlternateColor;



	}
}

