﻿using System;
using System.Linq;

using Hjg.Pngcs;

using Vintagestory.API.Client;
using Vintagestory.API.Common;
using Vintagestory.API.MathTools;

namespace Automap
{
	public class StandardRenderer : IChunkRenderer
	{
		private readonly int chunkSize;


		/// <summary>
		/// Renders shards similar to the Default VS version, plus P.O.I. markings.
		/// </summary>
		/// <param name="clientAPI">Client API.</param>
		/// <param name="logger">Logger.</param>
		public StandardRenderer(ICoreClientAPI clientAPI, ILogger logger) : base(clientAPI, logger)
		{
			chunkSize = ClientAPI.World.BlockAccessor.ChunkSize;
		}

		public override void GenerateChunkPngShard(Vec2i chunkPos, IMapChunk mc, ColumnMeta metaData, PngWriter pngWriter, out uint pixelCount)
		{
			pixelCount = 0;
			BlockPos tmpPos = new BlockPos();
			Vec2i localpos = new Vec2i();

			var chunksColumn = new IWorldChunk[ClientAPI.World.BlockAccessor.MapSizeY / chunkSize];

			int topChunkY = mc.YMax / chunkSize;//Heywaitaminute -- this isn't a highest FEATURE, if Rainmap isn't accurate!

			for (int chunkY = 0; chunkY <= topChunkY; chunkY++)
			{
				chunksColumn[chunkY] = ClientAPI.World.BlockAccessor.GetChunk(chunkPos.X, chunkY, chunkPos.Y);
				//What to do if chunk is a void? invalid?
			}

			// Prefetch map chunks, in pattern
			IMapChunk[] mapChunks = new IMapChunk[]
			{
			ClientAPI.World.BlockAccessor.GetMapChunk(chunkPos.X - 1, chunkPos.Y - 1),
			ClientAPI.World.BlockAccessor.GetMapChunk(chunkPos.X - 1, chunkPos.Y),
			ClientAPI.World.BlockAccessor.GetMapChunk(chunkPos.X, chunkPos.Y - 1)
			};

			//pre-create PNG line slices...
			ImageLine[] lines = Enumerable.Repeat(new object(), chunkSize).Select(l => new ImageLine(pngWriter.ImgInfo)).ToArray();

			for (int posIndex = 0; posIndex < (chunkSize * chunkSize); posIndex++)
			{
				int mapY = mc.RainHeightMap[posIndex];
				int localChunkY = mapY / chunkSize;
				if (localChunkY >= (chunksColumn.Length)) continue;//Out of range!

				MapUtil.PosInt2d(posIndex, chunkSize, localpos);
				int localX = localpos.X;
				int localZ = localpos.Y;

				float b = 1;
				int leftTop, rightTop, leftBot;

				IMapChunk leftTopMapChunk = mc;
				IMapChunk rightTopMapChunk = mc;
				IMapChunk leftBotMapChunk = mc;

				int topX = localX - 1;
				int botX = localX;
				int leftZ = localZ - 1;
				int rightZ = localZ;

				if (topX < 0 && leftZ < 0)
				{
					leftTopMapChunk = mapChunks[0];
					rightTopMapChunk = mapChunks[1];
					leftBotMapChunk = mapChunks[2];
				}
				else
				{
					if (topX < 0)
					{
						leftTopMapChunk = mapChunks[1];
						rightTopMapChunk = mapChunks[1];
					}
					if (leftZ < 0)
					{
						leftTopMapChunk = mapChunks[2];
						leftBotMapChunk = mapChunks[2];
					}
				}

				topX = GameMath.Mod(topX, chunkSize);
				leftZ = GameMath.Mod(leftZ, chunkSize);

				leftTop = leftTopMapChunk == null ? 0 : Math.Sign(mapY - leftTopMapChunk.RainHeightMap[leftZ * chunkSize + topX]);
				rightTop = rightTopMapChunk == null ? 0 : Math.Sign(mapY - rightTopMapChunk.RainHeightMap[rightZ * chunkSize + topX]);
				leftBot = leftBotMapChunk == null ? 0 : Math.Sign(mapY - leftBotMapChunk.RainHeightMap[leftZ * chunkSize + botX]);

				float slopeness = (leftTop + rightTop + leftBot);

				if (slopeness > 0) b = 1.2f;
				if (slopeness < 0) b = 0.8f;

				b -= 0.15f; //Slope boost value 

				if (chunksColumn[localChunkY] == null)
				{

					continue;
				}

				chunksColumn[localChunkY].Unpack();
				int blockId = chunksColumn[localChunkY].Blocks[MapUtil.Index3d(localpos.X, mapY % chunkSize, localpos.Y, chunkSize, chunkSize)];

				Block block = ClientAPI.World.Blocks[blockId];

				tmpPos.Set(chunkSize * chunkPos.X + localpos.X, mapY, chunkSize * chunkPos.Y + localpos.Y);

				int avgCol = block.GetColor(ClientAPI, tmpPos);
				int rndCol = block.GetRandomColor(ClientAPI, tmpPos, BlockFacing.UP);
				int col = ColorUtil.ColorOverlay(avgCol, rndCol, 0.125f);
				var packedFormat = ColorUtil.ColorMultiply3Clamped(col, b);

				int red = ColorUtil.ColorB(packedFormat);
				int green = ColorUtil.ColorG(packedFormat);
				int blue = ColorUtil.ColorR(packedFormat);


				//============ POI Population =================
				if (BlockID_Designators.ContainsKey(blockId))
				{
					var desig = BlockID_Designators[blockId];

					if (desig.Enabled)
					{
						red = desig.OverwriteColor.R;
						green = desig.OverwriteColor.G;
						blue = desig.OverwriteColor.B;
					}
				}

				ImageLineHelper.SetPixel(lines[localZ], localX, red, green, blue);

				//chunkImage.SetPixel(localX, localZ, pixelColor);
				pixelCount++;
			}

			for (int row = 0; row < pngWriter.ImgInfo.Rows; row++)
			{
				pngWriter.WriteRow(lines[row], row);
			}

			pngWriter.End();
		}
	}
}

